/*
 * Copyright (c) 2009 by Internet Systems Consortium, Inc. ("ISC")
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* Import. */

#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <nmsg.h>
#include <nmsg/pbmod.h>
#include <nmsg/pbmodset.h>
#include <nmsg/input.h>
#include <nmsg/isc/pbnmsg_isc_http.h>

#define DEBUG_LEVEL	0
#define MODULE_DIR	"/usr/local/lib/nmsg"

static nmsg_pbmod mod;

static void http_callback(Nmsg__NmsgPayload *, void *);

static void
http_callback(Nmsg__NmsgPayload *np, void *user __attribute__((unused))) {
	Nmsg__Isc__Http *http;
	char ts[64];
	char srcip[INET_ADDRSTRLEN];
	char dstip[INET_ADDRSTRLEN];
	char *request;
	struct tm *tm;
	time_t t;

	/* discard non-HTTP payloads */
	if (np->vid != NMSG_VENDOR_ISC_ID && np->msgtype != MSGTYPE_HTTP_ID)
		return;

	/* unpack HTTP nmsg payload */
	http = (Nmsg__Isc__Http *)
		protobuf_c_message_unpack(mod->pbdescr, NULL,
					  np->payload.len, np->payload.data);

	/* format timestamp */
	t = np->time_sec;
	tm = gmtime(&t);
	strftime(ts, sizeof(ts), "%F %T", tm);

	/* format srcip/dstip */
	srcip[0] = '\0';
	dstip[0] = '\0';
	if (http->has_srcip && http->srcip.len == 4)
		inet_ntop(AF_INET, http->srcip.data, srcip, INET_ADDRSTRLEN);
	if (http->has_dstip && http->dstip.len == 4)
		inet_ntop(AF_INET, http->dstip.data, dstip, INET_ADDRSTRLEN);

	/* extract HTTP request */
	if (http->has_request && http->request.data != NULL) {
		request = strtok((char *) http->request.data, "\n\r");
	} else {
		request = "";
	}
	if (request == NULL)
		request = "";

	/* output the special CSV format */
	printf("\"%s.%u\",\"%s\",\"%s\",\"%s\"\n",
	       ts, np->time_nsec, srcip, dstip, request);

	/* free unpacked HTTP nmsg payload */
	protobuf_c_message_free_unpacked((ProtobufCMessage *) http, NULL);
}

int
main(int argc, char **argv) {
	int fd;
	nmsg_buf buf;
	nmsg_pbmodset ms;
	nmsg_res res;
	void *clos;

	if (argc != 2) {
		fprintf(stderr, "usage: %s <NMSGfile>\n", argv[0]);
		return (1);
	}

	if (strcmp(argv[1], "-") != 0) {
		fd = open(argv[1], O_RDONLY);
		if (fd == -1) {
			perror("open");
			return (1);
		}
	} else {
		fd = STDIN_FILENO;
	}

	buf = nmsg_input_open_file(fd);
	if (buf == NULL) {
		fprintf(stderr, "nmsg_input_open_file() failed\n");
		return (1);
	}

	ms = nmsg_pbmodset_init(MODULE_DIR, DEBUG_LEVEL);
	if (ms == NULL) {
		fprintf(stderr, "nmsg_pbmodset_init() failed\n");
		return (1);
	}

	mod = nmsg_pbmodset_lookup(ms, NMSG_VENDOR_ISC_ID, MSGTYPE_HTTP_ID);
	if (mod == NULL) {
		fprintf(stderr, "unable to acquire module handle\n");
		return (1);
	}

	res = nmsg_pbmod_init(mod, &clos, DEBUG_LEVEL);
	if (res != nmsg_res_success) {
		fprintf(stderr, "unable to initialize module\n");
		return (res);
	}

	res = nmsg_input_loop(buf, -1, http_callback, NULL);
	if (res != nmsg_res_success && res != nmsg_res_eof) {
		fprintf(stderr, "nmsg_input_loop() returned %d\n", res);
		return (1);
	}

	nmsg_pbmod_fini(mod, &clos);
	nmsg_pbmodset_destroy(&ms);
	nmsg_input_close(&buf);

	return (0);
}
